#ifndef __ICD_NETWORK_GPRS_PRIVATE_H__
#define __ICD_NETWORK_GPRS_PRIVATE_H__

#include <gconf/gconf-client.h>
#include "icd/support/icd_dbus.h"
#include <icd/osso-ic-gconf.h>
#include "icd/network_api.h"

/* FIXME - move those defines to -dev package, fix connui-cellular as well */
#define ICD_GCONF_NETWORK_MAPPING_GPRS ICD_GCONF_NETWORK_MAPPING "/GPRS"

#define HOME_RX_BYTES ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_home_rx_bytes"
#define HOME_TX_BYTES ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_home_tx_bytes"
#define HOME_RST_TIME ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_home_reset_time"
#define HOME_WARNING_LIMIT ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_home_warning_limit"
#define HOME_NTFY_ENABLE ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_home_notification_enabled"
#define HOME_LAST_NTFY ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_home_last_notification"
#define HOME_NTFY_PERIOD ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_home_notification_period"

#define ROAM_RX_BYTES ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_roaming_rx_bytes"
#define ROAM_TX_BYTES ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_roaming_tx_bytes"
#define ROAM_RST_TIME ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_roaming_reset_time"
#define ROAM_WARNING_LIMIT ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_roaming_warning_limit"
#define ROAM_NTFY_ENABLE ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_roaming_notification_enabled"
#define ROAM_LAST_NTFY ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_roaming_last_notification"
#define ROAM_NTFY_PERIOD ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_roaming_notification_period"
#define ROAM_DISABLED ICD_GCONF_NETWORK_MAPPING_GPRS "/gprs_roaming_disabled"

enum observer_type
{
  OBSERVER_TYPE_SIM_IMSI = 0x0,
  OBSERVER_TYPE_NET_STATUS = 0x1,
  OBSERVER_TYPE_2 = 0x2,
  OBSERVER_TYPE_REG_STATUS = 0x3,
  OBSERVER_TYPE_ICD_UI = 0x4,
  OBSERVER_TYPE_5 = 0x5,
  OBSERVER_TYPE_6 = 0x6,
  OBSERVER_TYPE_APN = 0x7,
  OBSERVER_TYPE_PASSWORD = 0x8,
  OBSERVER_TYPE_USERNAME = 0x9,
  OBSERVER_TYPE_PDPADDRESS = 0xA,
  OBSERVER_TYPE_11 = 0xB,
  OBSERVER_TYPE_GPRS_PROPERTIES = 0xC,
  OBSERVER_TYPE_GPRS_13 = 0xD,
  OBSERVER_TYPE_DATACOUNTER = 0xE,
  OBSERVER_TYPE_LAST = 15
};

typedef struct
{
  GSList *listeners;
  DBusPendingCall *mcall;
  gboolean active;
} observer;

typedef struct
{
  guint suspension_timer;
  guint search_timeout_id;
  guint disconnect_timer;
  gboolean net_attached;
  gboolean net_available;
  guchar net_reg_status;
  guint sim_status;
  int online_status;
  gboolean emergency_call;
  GSList *networks;
  icd_nw_close_fn close_fn;
  icd_nw_watch_pid_fn watch_fn;
  gpointer watch_fn_token;
  DBusConnection *system_bus;
  GConfClient *gconf;
  gchar *imsi;
  icd_nw_search_cb_fn search_cb;
  gpointer search_cb_token;
  guint read_iaps_id;
  GSList* iap_dirs;
  guint dword50;
  int gap54;
  guint64 prev_gprs_tx_bytes;
  guint64 prev_gprs_rx_bytes;
  guint64 home_tx_bytes;
  guint64 home_rx_bytes;
  guint64 tx_bytes;
  guint64 rx_bytes;
  guint64 gprs_tx_bytes;
  guint64 gprs_rx_bytes;
  guint64 tx_bytes_prev;
  guint64 rx_bytes_prev;
  guint64 roaming_tx_bytes;
  guint64 roaming_rx_bytes;
  guint64 roaming_notification_period;
  guint64 home_notification_period;
  guint64 roaming_last_notification;
  guint64 home_last_notification;
  guint cnxn;
  DBusPendingCall *sim_request_imsi_pending;
  int imsi_requested;
  observer observers[OBSERVER_TYPE_LAST];
  int field_18C;
} gprs_private;

#endif /* __ICD_NETWORK_GPRS_PRIVATE_H__ */
