#include <icd/support/icd_log.h>

#include "gprs-private.h"
#include "observer.h"

typedef struct
{
  obsever_cb cb;
  gpointer user_data;
  gpointer token;
} observer_data;

gboolean
register_observer(enum observer_type type, obsever_cb cb, gprs_private *priv,
                  gpointer user_data, gpointer observer_token)
{
  observer_data *data;

  if (type >= G_N_ELEMENTS(priv->observers))
    return FALSE;

  data = g_new0(observer_data, 1);

  data->cb = cb;
  data->user_data = user_data;
  data->token = observer_token;
  priv->observers[type].listeners =
      g_slist_prepend(priv->observers[type].listeners, data);

  return TRUE;
}

gboolean
abort_observation(obsever_cb cb, gprs_private *priv, gpointer observer_token)
{
  enum observer_type type;

  for (type = 0; type < G_N_ELEMENTS(priv->observers); type++)
  {
    GSList *l = priv->observers[type].listeners;

    while (l)
    {
      observer_data *data = l->data;

      if (data->cb == cb && data->token == observer_token)
      {
        cb(type, OBSERVER_STATUS_ABORTED, priv, data->user_data);
        priv->observers[type].listeners =
            g_slist_remove(priv->observers[type].listeners, data);
        g_free(data);
        l = priv->observers[type].listeners;
      }
      else
        l = l->next;
    }
  }

  return TRUE;
}

gboolean
mark_observer_active(gprs_private *priv, enum observer_type type)
{
  observer *o = &priv->observers[type];

  if (!o->active)
  {
    o->mcall = NULL;
    o->active = TRUE;
  }

  return TRUE;
}

gboolean
notify_observers(enum observer_type type, enum observer_status status, gprs_private *priv)
{
  observer *observer;
  GSList *l;

  if (type >= OBSERVER_TYPE_LAST)
    return FALSE;

  ILOG_INFO("[GPRS] notify_observers: event %i has %i listeners", type,
            g_slist_length(priv->observers[type].listeners));

  observer = &priv->observers[type];
  observer->active = FALSE;
  observer->mcall = NULL;

  for (l = observer->listeners; l ; l = observer->listeners)
  {
    observer_data *data = l->data;

    if (data)
    {
      if (!data->cb(type, status, priv, data->user_data))
      {
        ILOG_INFO("[GPRS] An error was handled in the callback, aborting further observations for the callback");
        abort_observation(data->cb, priv, data->token);
        break;
      }
    }

    observer->listeners = g_slist_remove(observer->listeners, l);
  }

  return TRUE;
}
